@@ -25,9 +25,16 @@
 
 /**
  * SECTION:clutter-feature
- * @short_description: Query GL features at runtime
+ * @short_description: Run-time detection of Clutter features
  *
- * Functions to query available GL features ay runtime
+ * Parts of Clutter depend on the underlying platform, including the
+ * capabilities of the backend used and the OpenGL features exposed through the
+ * Clutter and COGL API.
+ *
+ * It is possible to ask whether Clutter has support for specific features at
+ * run-time.
+ *
+ * See also cogl_get_features() and #CoglFeatureFlags
  */
 
 #ifdef HAVE_CONFIG_H
@@ -95,6 +102,9 @@ _clutter_feature_init (void)
 
   context = _clutter_context_get_default ();
 
+  /* makes sure we have a GL context; if we have, this is a no-op */
+  _clutter_backend_create_context (context->backend, NULL);
+
   __features->flags = (_clutter_features_from_cogl (cogl_get_features ())
                     | _clutter_backend_get_features (context->backend));
 
