@@ -479,7 +479,9 @@ clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:15) */
+/* VOID:POINTER (./clutter-marshal.list:15) */
+
+/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:16) */
 void
 clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value G_GNUC_UNUSED,
@@ -518,7 +520,7 @@ clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./clutter-marshal.list:16) */
+/* VOID:STRING,INT (./clutter-marshal.list:17) */
 void
 clutter_marshal_VOID__STRING_INT (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -555,7 +557,46 @@ clutter_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT (./clutter-marshal.list:17) */
+/* VOID:UINT (./clutter-marshal.list:18) */
 
-/* VOID:VOID (./clutter-marshal.list:18) */
+/* VOID:VOID (./clutter-marshal.list:19) */
+
+/* VOID:STRING,INT,POINTER (./clutter-marshal.list:20) */
+void
+clutter_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
+                                          GValue       *return_value G_GNUC_UNUSED,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint G_GNUC_UNUSED,
+                                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING_INT_POINTER) (gpointer     data1,
+                                                         gpointer     arg_1,
+                                                         gint         arg_2,
+                                                         gpointer     arg_3,
+                                                         gpointer     data2);
+  register GMarshalFunc_VOID__STRING_INT_POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_INT_POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_pointer (param_values + 3),
+            data2);
+}
 
