@@ -67,6 +67,9 @@
 #include "clutter-stage-window.h"
 #include "clutter-version.h" 	/* For flavour */
 #include "clutter-id-pool.h"
+#include "clutter-container.h"
+#include "clutter-profile.h"
+#include "clutter-input-device.h"
 
 #include "cogl/cogl.h"
 
@@ -78,7 +81,9 @@ G_DEFINE_TYPE (ClutterStage, clutter_stage, CLUTTER_TYPE_GROUP);
 struct _ClutterStagePrivate
 {
   /* the stage implementation */
-  ClutterActor       *impl;
+  ClutterStageWindow *impl;
+  guint               minimum_width;
+  guint               minimum_height;
 
   ClutterColor        color;
   ClutterPerspective  perspective;
@@ -91,11 +96,11 @@ struct _ClutterStagePrivate
 
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
-  guint is_offscreen           : 1;
   guint is_cursor_visible      : 1;
   guint is_user_resizable      : 1;
   guint use_fog                : 1;
   guint throttle_motion_events : 1;
+  guint use_alpha              : 1;
 };
 
 enum
@@ -110,7 +115,9 @@ enum
   PROP_TITLE,
   PROP_USER_RESIZE,
   PROP_USE_FOG,
-  PROP_FOG
+  PROP_FOG,
+  PROP_USE_ALPHA,
+  PROP_KEY_FOCUS
 };
 
 enum
@@ -119,6 +126,7 @@ enum
   UNFULLSCREEN,
   ACTIVATE,
   DEACTIVATE,
+  DELETE_EVENT,
 
   LAST_SIGNAL
 };
@@ -134,14 +142,18 @@ clutter_stage_get_preferred_width (ClutterActor *self,
                                    gfloat       *natural_width_p)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  ClutterGeometry geom = { 0, };
 
   if (priv->impl == NULL)
     return;
 
-  CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_width (priv->impl,
-                                                             for_height,
-                                                             min_width_p,
-                                                             natural_width_p);
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
+
+  if (min_width_p)
+    *min_width_p = geom.width;
+
+  if (natural_width_p)
+    *natural_width_p = geom.width;
 }
 
 static void
@@ -151,14 +163,18 @@ clutter_stage_get_preferred_height (ClutterActor *self,
                                     gfloat       *natural_height_p)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  ClutterGeometry geom = { 0, };
 
   if (priv->impl == NULL)
     return;
 
-  CLUTTER_ACTOR_GET_CLASS (priv->impl)->get_preferred_height (priv->impl,
-                                                              for_width,
-                                                              min_height_p,
-                                                              natural_height_p);
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
+
+  if (min_height_p)
+    *min_height_p = geom.height;
+
+  if (natural_height_p)
+    *natural_height_p = geom.height;
 }
 static void
 clutter_stage_allocate (ClutterActor           *self,
@@ -167,12 +183,16 @@ clutter_stage_allocate (ClutterActor           *self,
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
   gboolean origin_changed;
+  gint width, height;
 
   origin_changed = (flags & CLUTTER_ABSOLUTE_ORIGIN_CHANGED) ? TRUE : FALSE;
 
   if (priv->impl == NULL)
     return;
 
+  width = clutter_actor_box_get_width (box);
+  height = clutter_actor_box_get_height (box);
+
   /* if the stage is fixed size (for instance, it's using a frame-buffer)
    * then we simply ignore any allocation request and override the
    * allocation chain.
@@ -181,43 +201,41 @@ clutter_stage_allocate (ClutterActor           *self,
     {
       ClutterActorClass *klass;
 
+      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
+       * need to manually keep it informed of the current window size */
+      _cogl_onscreen_clutter_backend_set_size (width, height);
+
       CLUTTER_NOTE (LAYOUT,
                     "Following allocation to %dx%d (origin %s)",
-                    (int) (box->x2 - box->x1),
-                    (int) (box->y2 - box->y1),
+                    width, height,
                     origin_changed ? "changed" : "not changed");
 
       klass = CLUTTER_ACTOR_CLASS (clutter_stage_parent_class);
       klass->allocate (self, box, flags);
 
-      klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
-      klass->allocate (priv->impl, box, flags);
+      _clutter_stage_window_resize (priv->impl, width, height);
     }
   else
     {
       ClutterActorBox override = { 0, };
+      ClutterGeometry geom = { 0, };
       ClutterActorClass *klass;
-      gfloat natural_width, natural_height;
 
-      /* propagate the allocation */
-      klass = CLUTTER_ACTOR_GET_CLASS (priv->impl);
-      klass->allocate (self, box, flags);
+      _clutter_stage_window_get_geometry (priv->impl, &geom);
 
-      /* get the preferred size from the backend */
-      clutter_actor_get_preferred_size (priv->impl,
-                                        NULL, NULL,
-                                        &natural_width, &natural_height);
+      /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
+       * need to manually keep it informed of the current window size */
+      _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
 
       override.x1 = 0;
       override.y1 = 0;
-      override.x2 = natural_width;
-      override.y2 = natural_height;
+      override.x2 = geom.width;
+      override.y2 = geom.height;
 
       CLUTTER_NOTE (LAYOUT,
                     "Overrigin original allocation of %dx%d "
                     "with %dx%d (origin %s)",
-                    (int) (box->x2 - box->x1),
-                    (int) (box->y2 - box->y1),
+                    width, height,
                     (int) (override.x2),
                     (int) (override.y2),
                     origin_changed ? "changed" : "not changed");
@@ -233,17 +251,36 @@ clutter_stage_paint (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
   CoglColor stage_color;
+  guint8 real_alpha;
+  CLUTTER_STATIC_TIMER (stage_clear_timer,
+                        "Painting actors", /* parent */
+                        "Stage clear",
+                        "The time spent clearing the stage",
+                        0 /* no application private data */);
 
   CLUTTER_NOTE (PAINT, "Initializing stage paint");
 
+  /* composite the opacity to the stage color */
+  real_alpha = clutter_actor_get_opacity (self)
+             * priv->color.alpha
+             / 255;
+
+  /* we use the real alpha to clear the stage if :use-alpha is
+   * set; the effect depends entirely on how the Clutter backend
+   */
   cogl_color_set_from_4ub (&stage_color,
                            priv->color.red,
                            priv->color.green,
                            priv->color.blue,
-                           priv->color.alpha);
+                           priv->use_alpha ? real_alpha
+                                           : 255);
+  cogl_color_premultiply (&stage_color);
+
+  CLUTTER_TIMER_START (_clutter_uprof_context, stage_clear_timer);
   cogl_clear (&stage_color,
 	      COGL_BUFFER_BIT_COLOR |
 	      COGL_BUFFER_BIT_DEPTH);
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, stage_clear_timer);
 
   if (priv->use_fog)
     {
@@ -260,9 +297,6 @@ clutter_stage_paint (ClutterActor *self)
   else
     cogl_disable_fog ();
 
-  CLUTTER_NOTE (PAINT, "Proxying the paint to the stage implementation");
-  clutter_actor_paint (priv->impl);
-
   /* this will take care of painting every child */
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->paint (self);
 }
@@ -271,18 +305,20 @@ static void
 clutter_stage_pick (ClutterActor       *self,
 		    const ClutterColor *color)
 {
-  /* Paint nothing, cogl_paint_init() effectively paints the stage
-   * silhouette for us - see _clutter_do_pick().
-   * Chain up to the groups paint howerer so our children get picked
-   * - clutter_group_pick
+  /* Note: we don't chain up to our parent as we don't want any geometry
+   * emitted for the stage itself. The stage's pick id is effectively handled
+   * by the call to cogl_clear done in clutter-main.c:_clutter_do_pick_async()
    */
-  CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->pick (self, color);
+  clutter_container_foreach (CLUTTER_CONTAINER (self),
+                             CLUTTER_CALLBACK (clutter_actor_paint),
+                             NULL);
 }
 
 static void
 clutter_stage_realize (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
+  gboolean is_realized;
 
   /* Make sure the viewport and projection matrix are valid for the
    * first paint (which will likely occur before the ConfigureNotify
@@ -291,13 +327,34 @@ clutter_stage_realize (ClutterActor *self)
   CLUTTER_SET_PRIVATE_FLAGS (self, CLUTTER_ACTOR_SYNC_MATRICES);
 
   g_assert (priv->impl != NULL);
-  clutter_actor_realize (priv->impl);
+  is_realized = _clutter_stage_window_realize (priv->impl);
 
   /* ensure that the stage is using the context if the
    * realization sequence was successful
    */
-  if (CLUTTER_ACTOR_IS_REALIZED (priv->impl))
-    clutter_stage_ensure_current (CLUTTER_STAGE (self));
+  if (is_realized)
+    {
+      ClutterBackend *backend = clutter_get_default_backend ();
+      GError *error = NULL;
+
+      /* We want to select the context without calling
+         clutter_backend_ensure_context so that it doesn't call any
+         Cogl functions. Otherwise it would create the Cogl context
+         before we get a chance to check whether the GL version is
+         valid */
+      _clutter_backend_ensure_context_internal (backend, CLUTTER_STAGE (self));
+
+      /* Make sure Cogl can support the driver */
+      if (!_cogl_check_driver_valid (&error))
+        {
+          g_critical ("The GL driver is not supported: %s",
+                      error->message);
+          g_clear_error (&error);
+          CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+        }
+      else
+        CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+    }
   else
     CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
 }
@@ -309,7 +366,9 @@ clutter_stage_unrealize (ClutterActor *self)
 
   /* and then unrealize the implementation */
   g_assert (priv->impl != NULL);
-  clutter_actor_unrealize (priv->impl);
+  _clutter_stage_window_unrealize (priv->impl);
+
+  CLUTTER_ACTOR_UNSET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
 
   clutter_stage_ensure_current (CLUTTER_STAGE (self));
 }
@@ -318,7 +377,6 @@ static void
 clutter_stage_show (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
-  ClutterStageWindow *impl;
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->show (self);
 
@@ -327,19 +385,16 @@ clutter_stage_show (ClutterActor *self)
   _clutter_stage_maybe_relayout (self);
 
   g_assert (priv->impl != NULL);
-  impl = CLUTTER_STAGE_WINDOW (priv->impl);
-  CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->show (impl, TRUE);
+  _clutter_stage_window_show (priv->impl, TRUE);
 }
 
 static void
 clutter_stage_hide (ClutterActor *self)
 {
   ClutterStagePrivate *priv = CLUTTER_STAGE (self)->priv;
-  ClutterStageWindow *impl;
 
   g_assert (priv->impl != NULL);
-  impl = CLUTTER_STAGE_WINDOW (priv->impl);
-  CLUTTER_STAGE_WINDOW_GET_IFACE (impl)->hide (impl);
+  _clutter_stage_window_hide (priv->impl);
 
   CLUTTER_ACTOR_CLASS (clutter_stage_parent_class)->hide (self);
 }
@@ -375,7 +430,7 @@ static void
 clutter_stage_real_fullscreen (ClutterStage *stage)
 {
   ClutterStagePrivate *priv = stage->priv;
-  gfloat natural_width, natural_height;
+  ClutterGeometry geom;
   ClutterActorBox box;
 
   /* we need to force an allocation here because the size
@@ -385,14 +440,12 @@ clutter_stage_real_fullscreen (ClutterStage *stage)
    * the fact that fullscreening the stage on the X11 backends
    * is really an asynchronous operation
    */
-  clutter_actor_get_preferred_size (CLUTTER_ACTOR (priv->impl),
-                                    NULL, NULL,
-                                    &natural_width, &natural_height);
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
 
   box.x1 = 0;
   box.y1 = 0;
-  box.x2 = natural_width;
-  box.y2 = natural_height;
+  box.x2 = geom.width;
+  box.y2 = geom.height;
 
   clutter_actor_allocate (CLUTTER_ACTOR (stage),
                           &box,
@@ -405,6 +458,7 @@ _clutter_stage_queue_event (ClutterStage *stage,
 {
   ClutterStagePrivate *priv;
   gboolean first_event;
+  ClutterInputDevice *device;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
@@ -412,14 +466,31 @@ _clutter_stage_queue_event (ClutterStage *stage,
 
   first_event = priv->event_queue->length == 0;
 
-  g_queue_push_tail (priv->event_queue,
-		     clutter_event_copy (event));
+  g_queue_push_tail (priv->event_queue, clutter_event_copy (event));
 
   if (first_event)
     {
       ClutterMasterClock *master_clock = _clutter_master_clock_get_default ();
       _clutter_master_clock_start_running (master_clock);
     }
+
+  /* if needed, update the state of the input device of the event.
+   * we do it here to avoid calling the same code from every backend
+   * event processing function
+   */
+  device = clutter_event_get_device (event);
+  if (device != NULL)
+    {
+      ClutterModifierType event_state = clutter_event_get_state (event);
+      guint32 event_time = clutter_event_get_time (event);
+      gfloat event_x, event_y;
+
+      clutter_event_get_coords (event, &event_x, &event_y);
+
+      _clutter_input_device_set_coords (device, event_x, event_y);
+      _clutter_input_device_set_state (device, event_state);
+      _clutter_input_device_set_time (device, event_time);
+    }
 }
 
 gboolean
@@ -438,7 +509,7 @@ void
 _clutter_stage_process_queued_events (ClutterStage *stage)
 {
   ClutterStagePrivate *priv;
-  GList *events, *l;;
+  GList *events, *l;
 
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
@@ -457,7 +528,7 @@ _clutter_stage_process_queued_events (ClutterStage *stage)
   priv->event_queue->tail = NULL;
   priv->event_queue->length = 0;
 
-  for (l = events; l; l = l->next)
+  for (l = events; l != NULL; l = l->next)
     {
       ClutterEvent *event;
       ClutterEvent *next_event;
@@ -577,11 +648,16 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
     CLUTTER_CONTEXT ()->redraw_count += 1;
 }
 
-static void
-set_offscreen_while_unrealized (ClutterActor *actor,
-                                void         *data)
+static gboolean
+clutter_stage_real_delete_event (ClutterStage *stage,
+                                 ClutterEvent *event)
 {
-  CLUTTER_STAGE (actor)->priv->is_offscreen = GPOINTER_TO_INT (data);
+  if (clutter_stage_is_default (stage))
+    clutter_main_quit ();
+  else
+    clutter_actor_destroy (CLUTTER_ACTOR (stage));
+
+  return TRUE;
 }
 
 static void
@@ -605,26 +681,8 @@ clutter_stage_set_property (GObject      *object,
       break;
 
     case PROP_OFFSCREEN:
-      {
-        gboolean was_showing;
-
-        if (priv->is_offscreen == g_value_get_boolean (value))
-          return;
-
-        was_showing = CLUTTER_ACTOR_IS_VISIBLE (actor);
-
-        /* Backend needs to check this prop and handle accordingly
-         * in realise.
-         * FIXME: More 'obvious' implementation needed?
-         */
-        _clutter_actor_rerealize (actor,
-                                  set_offscreen_while_unrealized,
-                                  GINT_TO_POINTER (g_value_get_boolean (value)));
-
-        if (was_showing &&
-            !CLUTTER_ACTOR_IS_REALIZED (actor))
-          priv->is_offscreen = ~g_value_get_boolean (value);
-      }
+      if (g_value_get_boolean (value))
+        g_warning ("Offscreen stages are currently not supported\n");
       break;
 
     case PROP_CURSOR_VISIBLE:
@@ -654,6 +712,14 @@ clutter_stage_set_property (GObject      *object,
       clutter_stage_set_fog (stage, g_value_get_boxed (value));
       break;
 
+    case PROP_USE_ALPHA:
+      clutter_stage_set_use_alpha (stage, g_value_get_boolean (value));
+      break;
+
+    case PROP_KEY_FOCUS:
+      clutter_stage_set_key_focus (stage, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -675,7 +741,7 @@ clutter_stage_get_property (GObject    *gobject,
       break;
 
     case PROP_OFFSCREEN:
-      g_value_set_boolean (value, priv->is_offscreen);
+      g_value_set_boolean (value, FALSE);
       break;
 
     case PROP_FULLSCREEN_SET:
@@ -706,6 +772,14 @@ clutter_stage_get_property (GObject    *gobject,
       g_value_set_boxed (value, &priv->fog);
       break;
 
+    case PROP_USE_ALPHA:
+      g_value_set_boolean (value, priv->use_alpha);
+      break;
+
+    case PROP_KEY_FOCUS:
+      g_value_set_object (value, priv->key_focused_actor);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -718,16 +792,34 @@ clutter_stage_dispose (GObject *object)
   ClutterStage        *stage = CLUTTER_STAGE (object);
   ClutterStagePrivate *priv = stage->priv;
   ClutterStageManager *stage_manager = clutter_stage_manager_get_default ();
+  ClutterMainContext  *context;
+  GList               *l, *next;
 
   clutter_actor_hide (CLUTTER_ACTOR (object));
 
   _clutter_stage_manager_remove_stage (stage_manager, stage);
 
-  if (priv->impl)
+  context = _clutter_context_get_default ();
+
+  /* Remove any pending events for this stage from the event queue */
+  for (l = context->events_queue->head; l; l = next)
+    {
+      ClutterEvent *event = l->data;
+
+      next = l->next;
+
+      if (event->any.stage == stage)
+        {
+          g_queue_delete_link (context->events_queue, l);
+          clutter_event_free (event);
+        }
+    }
+
+  if (priv->impl != NULL)
     {
       CLUTTER_NOTE (BACKEND, "Disposing of the stage implementation");
-      clutter_actor_hide (priv->impl);
-      clutter_actor_destroy (priv->impl);
+
+      g_object_unref (priv->impl);
       priv->impl = NULL;
     }
 
@@ -748,7 +840,6 @@ clutter_stage_finalize (GObject *object)
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
-
 static void
 clutter_stage_class_init (ClutterStageClass *klass)
 {
@@ -915,6 +1006,41 @@ clutter_stage_class_init (ClutterStageClass *klass)
                               CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_FOG, pspec);
 
+  /**
+   * ClutterStage:use-alpha:
+   *
+   * Whether the #ClutterStage should honour the alpha component of the
+   * #ClutterStage:color property when painting. If Clutter is run under
+   * a compositing manager this will result in the stage being blended
+   * with the underlying window(s)
+   *
+   * Since: 1.2
+   */
+  pspec = g_param_spec_boolean ("use-alpha",
+                                "Use Alpha",
+                                "Whether to honour the alpha component of "
+                                "the stage color",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_USE_ALPHA, pspec);
+
+  /**
+   * ClutterStage:key-focus:
+   *
+   * The #ClutterActor that will receive key events from the underlying
+   * windowing system.
+   *
+   * If %NULL, the #ClutterStage will receive the events.
+   *
+   * Since: 1.2
+   */
+  pspec = g_param_spec_object ("key-focus",
+                               "Key Focus",
+                               "The currently key focused actor",
+                               CLUTTER_TYPE_ACTOR,
+                               CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_KEY_FOCUS, pspec);
+
   /**
    * ClutterStage::fullscreen
    * @stage: the stage which was fullscreened
@@ -983,9 +1109,41 @@ clutter_stage_class_init (ClutterStageClass *klass)
 		  clutter_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
+  /**
+   * ClutterStage::delete-event:
+   * @stage: the stage that received the event
+   * @event: a #ClutterEvent of type %CLUTTER_DELETE
+   *
+   * The ::delete-event signal is emitted when the user closes a
+   * #ClutterStage window using the window controls.
+   *
+   * Clutter by default will call clutter_main_quit() if @stage is
+   * the default stage, and clutter_actor_destroy() for any other
+   * stage.
+   *
+   * It is possible to override the default behaviour by connecting
+   * a new handler and returning %TRUE there.
+   *
+   * <note>This signal is emitted only on Clutter backends that
+   * embed #ClutterStage in native windows. It is not emitted for
+   * backends that use a static frame buffer.</note>
+   *
+   * Since: 1.2
+   */
+  stage_signals[DELETE_EVENT] =
+    g_signal_new (I_("delete-event"),
+                  G_TYPE_FROM_CLASS (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (ClutterStageClass, delete_event),
+                  _clutter_boolean_handled_accumulator, NULL,
+                  clutter_marshal_BOOLEAN__BOXED,
+                  G_TYPE_BOOLEAN, 1,
+                  CLUTTER_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   klass->fullscreen = clutter_stage_real_fullscreen;
   klass->activate = clutter_stage_real_activate;
   klass->deactivate = clutter_stage_real_deactivate;
+  klass->delete_event = clutter_stage_real_delete_event;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterStagePrivate));
 }
@@ -1008,20 +1166,14 @@ clutter_stage_init (ClutterStage *self)
     {
       g_warning ("Unable to create a new stage, falling back to the "
                  "default stage.");
-      priv->impl = CLUTTER_ACTOR (_clutter_stage_get_default_window ());
+      priv->impl = _clutter_stage_get_default_window ();
 
       /* at this point we must have a default stage, or we're screwed */
       g_assert (priv->impl != NULL);
     }
-  else
-    g_object_ref_sink (priv->impl);
-
-  /* make sure that the implementation is considered a top level */
-  CLUTTER_SET_PRIVATE_FLAGS (priv->impl, CLUTTER_ACTOR_IS_TOPLEVEL);
 
   priv->event_queue = g_queue_new ();
 
-  priv->is_offscreen           = FALSE;
   priv->is_fullscreen          = FALSE;
   priv->is_user_resizable      = FALSE;
   priv->is_cursor_visible      = TRUE;
@@ -1029,6 +1181,8 @@ clutter_stage_init (ClutterStage *self)
   priv->throttle_motion_events = TRUE;
 
   priv->color = default_stage_color;
+  priv->minimum_width = 1;
+  priv->minimum_height = 1;
 
   priv->perspective.fovy   = 60.0; /* 60 Degrees */
   priv->perspective.aspect = 1.0;
@@ -1040,6 +1194,7 @@ clutter_stage_init (ClutterStage *self)
   priv->fog.z_far  = 2.0;
 
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
+  clutter_stage_set_title (self, g_get_prgname ());
   clutter_stage_set_key_focus (self, NULL);
 }
 
@@ -1064,11 +1219,16 @@ clutter_stage_get_default (void)
 
   stage = clutter_stage_manager_get_default_stage (stage_manager);
   if (G_UNLIKELY (stage == NULL))
-    /* This will take care of automatically adding the stage to the
-     * stage manager and setting it as the default. Its floating
-     * reference will be claimed by the stage manager.
-     */
-    stage = g_object_new (CLUTTER_TYPE_STAGE, NULL);
+    {
+      /* This will take care of automatically adding the stage to the
+       * stage manager and setting it as the default. Its floating
+       * reference will be claimed by the stage manager.
+       */
+      stage = g_object_new (CLUTTER_TYPE_STAGE, NULL);
+
+      /* the default stage is realized by default */
+      clutter_actor_realize (CLUTTER_ACTOR (stage));
+    }
 
   return CLUTTER_ACTOR (stage);
 }
@@ -1470,6 +1630,9 @@ clutter_stage_event (ClutterStage *stage,
 
       g_signal_emit_by_name (stage, "event", event, &retval);
 
+      if (!retval)
+        g_signal_emit_by_name (stage, "delete-event", event, &retval);
+
       return retval;
     }
 
@@ -1580,7 +1743,7 @@ on_key_focused_weak_notify (gpointer data,
 /**
  * clutter_stage_set_key_focus:
  * @stage: the #ClutterStage
- * @actor: the actor to set key focus to, or %NULL
+ * @actor: (allow-none): the actor to set key focus to, or %NULL
  *
  * Sets the key focus on @actor. An actor with key focus will receive
  * all the key events. If @actor is %NULL, the stage will receive
@@ -1641,6 +1804,8 @@ clutter_stage_set_key_focus (ClutterStage *stage,
     }
   else
     g_signal_emit_by_name (stage, "key-focus-in");
+
+  g_object_notify (G_OBJECT (stage), "key-focus");
 }
 
 /**
@@ -2044,7 +2209,7 @@ _clutter_stage_set_window (ClutterStage       *stage,
   if (stage->priv->impl)
     g_object_unref (stage->priv->impl);
 
-  stage->priv->impl = CLUTTER_ACTOR (stage_window);
+  stage->priv->impl = stage_window;
 }
 
 ClutterStageWindow *
@@ -2111,3 +2276,124 @@ clutter_stage_get_throttle_motion_events (ClutterStage *stage)
 
   return stage->priv->throttle_motion_events;
 }
+
+/**
+ * clutter_stage_set_use_alpha:
+ * @stage: a #ClutterStage
+ * @use_alpha: whether the stage should honour the opacity or the
+ *   alpha channel of the stage color
+ *
+ * Sets whether the @stage should honour the #ClutterActor:opacity and
+ * the alpha channel of the #ClutterStage:color
+ *
+ * Since: 1.2
+ */
+void
+clutter_stage_set_use_alpha (ClutterStage *stage,
+                             gboolean      use_alpha)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  priv = stage->priv;
+
+  if (priv->use_alpha != use_alpha)
+    {
+      priv->use_alpha = use_alpha;
+
+      clutter_actor_queue_redraw (CLUTTER_ACTOR (stage));
+
+      g_object_notify (G_OBJECT (stage), "use-alpha");
+    }
+}
+
+/**
+ * clutter_stage_get_use_alpha:
+ * @stage: a #ClutterStage
+ *
+ * Retrieves the value set using clutter_stage_set_use_alpha()
+ *
+ * Return value: %TRUE if the stage should honour the opacity and the
+ *   alpha channel of the stage color
+ *
+ * Since: 1.2
+ */
+gboolean
+clutter_stage_get_use_alpha (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
+
+  return stage->priv->use_alpha;
+}
+
+/**
+ * clutter_stage_set_minimum_size:
+ * @stage: a #ClutterStage
+ * @width: width, in pixels
+ * @height: height, in pixels
+ *
+ * Sets the minimum size for a stage window. This has no effect if the stage
+ * is fullscreen.
+ *
+ * Since: 1.2
+ */
+void
+clutter_stage_set_minimum_size (ClutterStage *stage,
+                                guint         width,
+                                guint         height)
+{
+  gboolean resize;
+  ClutterGeometry geom;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+  g_return_if_fail ((width > 0) && (height > 0));
+
+  stage->priv->minimum_width = width;
+  stage->priv->minimum_height = height;
+
+  if (stage->priv->impl == NULL)
+    return;
+
+  resize = FALSE;
+  _clutter_stage_window_get_geometry (stage->priv->impl, &geom);
+
+  if (geom.width < width)
+    resize = TRUE;
+  else
+    width = geom.width;
+
+  if (geom.height < height)
+    resize = TRUE;
+  else
+    height = geom.height;
+
+  if (resize)
+    _clutter_stage_window_resize (stage->priv->impl, width, height);
+}
+
+/**
+ * clutter_stage_get_minimum_size:
+ * @stage: a #ClutterStage
+ * @width: width, in pixels
+ * @height: height, in pixels
+ *
+ * Gets the set minimum size for a stage window. This may not correspond
+ * to the actual minimum size and is specific to the back-end
+ * implementation.
+ *
+ * Since: 1.2
+ */
+void
+clutter_stage_get_minimum_size (ClutterStage *stage,
+                                guint        *width,
+                                guint        *height)
+{
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  if (width)
+    *width = stage->priv->minimum_width;
+  if (height)
+    *height = stage->priv->minimum_height;
+}
+
