@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,7 +34,7 @@
 /* TO rgba */
 
 inline static void
-_cogl_g_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_g_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[0];
   dst[1] = src[0];
@@ -43,7 +43,7 @@ _cogl_g_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgb_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_rgb_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[0];
   dst[1] = src[1];
@@ -52,7 +52,7 @@ _cogl_rgb_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_bgr_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_bgr_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[2];
   dst[1] = src[1];
@@ -61,7 +61,7 @@ _cogl_bgr_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_bgra_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_bgra_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[2];
   dst[1] = src[1];
@@ -70,7 +70,7 @@ _cogl_bgra_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_argb_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_argb_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[1];
   dst[1] = src[2];
@@ -79,7 +79,7 @@ _cogl_argb_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_abgr_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_abgr_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[3];
   dst[1] = src[2];
@@ -88,7 +88,7 @@ _cogl_abgr_to_rgba (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgba_to_rgba (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_rgba (const guchar *src, guchar *dst)
 {
   dst[0] = src[0];
   dst[1] = src[1];
@@ -99,13 +99,13 @@ _cogl_rgba_to_rgba (const guint8 *src, guint8 *dst)
 /* FROM rgba */
 
 inline static void
-_cogl_rgba_to_g (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_g (const guchar *src, guchar *dst)
 {
   dst[0] = (src[0] + src[1] + src[2]) / 3;
 }
 
 inline static void
-_cogl_rgba_to_rgb (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_rgb (const guchar *src, guchar *dst)
 {
   dst[0] = src[0];
   dst[1] = src[1];
@@ -113,7 +113,7 @@ _cogl_rgba_to_rgb (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgba_to_bgr (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_bgr (const guchar *src, guchar *dst)
 {
   dst[0] = src[2];
   dst[1] = src[1];
@@ -121,7 +121,7 @@ _cogl_rgba_to_bgr (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgba_to_bgra (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_bgra (const guchar *src, guchar *dst)
 {
   dst[0] = src[2];
   dst[1] = src[1];
@@ -130,7 +130,7 @@ _cogl_rgba_to_bgra (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgba_to_argb (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_argb (const guchar *src, guchar *dst)
 {
   dst[0] = src[3];
   dst[1] = src[0];
@@ -139,7 +139,7 @@ _cogl_rgba_to_argb (const guint8 *src, guint8 *dst)
 }
 
 inline static void
-_cogl_rgba_to_abgr (const guint8 *src, guint8 *dst)
+_cogl_rgba_to_abgr (const guchar *src, guchar *dst)
 {
   dst[0] = src[3];
   dst[1] = src[2];
@@ -150,7 +150,7 @@ _cogl_rgba_to_abgr (const guint8 *src, guint8 *dst)
 /* (Un)Premultiplication */
 
 inline static void
-_cogl_unpremult_alpha_0 (guint8 *dst)
+_cogl_unpremult_alpha_0 (guchar *dst)
 {
   dst[0] = 0;
   dst[1] = 0;
@@ -159,9 +159,9 @@ _cogl_unpremult_alpha_0 (guint8 *dst)
 }
 
 inline static void
-_cogl_unpremult_alpha_last (guint8 *dst)
+_cogl_unpremult_alpha_last (guchar *dst)
 {
-  guint8 alpha = dst[3];
+  guchar alpha = dst[3];
 
   dst[0] = (dst[0] * 255) / alpha;
   dst[1] = (dst[1] * 255) / alpha;
@@ -169,9 +169,9 @@ _cogl_unpremult_alpha_last (guint8 *dst)
 }
 
 inline static void
-_cogl_unpremult_alpha_first (guint8 *dst)
+_cogl_unpremult_alpha_first (guchar *dst)
 {
-  guint8 alpha = dst[0];
+  guchar alpha = dst[0];
 
   dst[1] = (dst[1] * 255) / alpha;
   dst[2] = (dst[2] * 255) / alpha;
@@ -189,24 +189,24 @@ _cogl_unpremult_alpha_first (guint8 *dst)
   } G_STMT_END
 
 inline static void
-_cogl_premult_alpha_last (guint8 *dst)
+_cogl_premult_alpha_last (guchar *dst)
 {
-  guint8 alpha = dst[3];
+  guchar alpha = dst[3];
   /* Using a separate temporary per component has given slightly better
    * code generation with GCC in the past; it shouldn't do any worse in
    * any case.
    */
-  unsigned int t1, t2, t3;
+  guint t1, t2, t3;
   MULT(dst[0], alpha, t1);
   MULT(dst[1], alpha, t2);
   MULT(dst[2], alpha, t3);
 }
 
 inline static void
-_cogl_premult_alpha_first (guint8 *dst)
+_cogl_premult_alpha_first (guchar *dst)
 {
-  guint8 alpha = dst[0];
-  unsigned int t1, t2, t3;
+  guchar alpha = dst[0];
+  guint t1, t2, t3;
 
   MULT(dst[1], alpha, t1);
   MULT(dst[2], alpha, t2);
@@ -215,91 +215,6 @@ _cogl_premult_alpha_first (guint8 *dst)
 
 #undef MULT
 
-/* Use the SSE optimized version to premult four pixels at once when
-   it is available. The same assembler code works for x86 and x86-64
-   because it doesn't refer to any non-SSE registers directly */
-#if defined(__SSE2__) && defined(__GNUC__) \
-  && (defined(__x86_64) || defined(__i386))
-#define COGL_USE_PREMULT_SSE2
-#endif
-
-#ifdef COGL_USE_PREMULT_SSE2
-
-inline static void
-_cogl_premult_alpha_last_four_pixels_sse2 (guint8 *p)
-{
-  /* 8 copies of 128 used below */
-  static const gint16 eight_halves[8] __attribute__ ((aligned (16))) =
-    { 128, 128, 128, 128, 128, 128, 128, 128 };
-  /* Mask of the rgb components of the four pixels */
-  static const gint8 just_rgb[16] __attribute__ ((aligned (16))) =
-    { 0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0x00,
-      0xff, 0xff, 0xff, 0x00, 0xff, 0xff, 0xff, 0x00 };
-  /* Each SSE register only holds two pixels because we need to work
-     with 16-bit intermediate values. We still do four pixels by
-     interleaving two registers in the hope that it will pipeline
-     better */
-  asm (/* Load eight_halves into xmm5 for later */
-       "movdqa (%1), %%xmm5\n"
-       /* Clear xmm3 */
-       "pxor %%xmm3, %%xmm3\n"
-       /* Load two pixels from p into the low half of xmm0 */
-       "movlps (%0), %%xmm0\n"
-       /* Load the next set of two pixels from p into the low half of xmm1 */
-       "movlps 8(%0), %%xmm1\n"
-       /* Unpack 8 bytes from the low quad-words in each register to 8
-          16-bit values */
-       "punpcklbw %%xmm3, %%xmm0\n"
-       "punpcklbw %%xmm3, %%xmm1\n"
-       /* Copy alpha values of the first pixel in xmm0 to all
-          components of the first pixel in xmm2 */
-       "pshuflw $255, %%xmm0, %%xmm2\n"
-       /* same for xmm1 and xmm3 */
-       "pshuflw $255, %%xmm1, %%xmm3\n"
-       /* The above also copies the second pixel directly so we now
-          want to replace the RGB components with copies of the alpha
-          components */
-       "pshufhw $255, %%xmm2, %%xmm2\n"
-       "pshufhw $255, %%xmm3, %%xmm3\n"
-       /* Multiply the rgb components by the alpha */
-       "pmullw %%xmm2, %%xmm0\n"
-       "pmullw %%xmm3, %%xmm1\n"
-       /* Add 128 to each component */
-       "paddw %%xmm5, %%xmm0\n"
-       "paddw %%xmm5, %%xmm1\n"
-       /* Copy the results to temporary registers xmm4 and xmm5 */
-       "movdqa %%xmm0, %%xmm4\n"
-       "movdqa %%xmm1, %%xmm5\n"
-       /* Divide the results by 256 */
-       "psrlw $8, %%xmm0\n"
-       "psrlw $8, %%xmm1\n"
-       /* Add the temporaries back in */
-       "paddw %%xmm4, %%xmm0\n"
-       "paddw %%xmm5, %%xmm1\n"
-       /* Divide again */
-       "psrlw $8, %%xmm0\n"
-       "psrlw $8, %%xmm1\n"
-       /* Pack the results back as bytes */
-       "packuswb %%xmm1, %%xmm0\n"
-       /* Load just_rgb into xmm3 for later */
-       "movdqa (%2), %%xmm3\n"
-       /* Reload all four pixels into xmm2 */
-       "movups (%0), %%xmm2\n"
-       /* Mask out the alpha from the results */
-       "andps %%xmm3, %%xmm0\n"
-       /* Mask out the RGB from the original four pixels */
-       "andnps %%xmm2, %%xmm3\n"
-       /* Combine the two to get the right alpha values */
-       "orps %%xmm3, %%xmm0\n"
-       /* Write to memory */
-       "movdqu %%xmm0, (%0)\n"
-       : /* no outputs */
-       : "r" (p), "r" (eight_halves), "r" (just_rgb)
-       : "xmm0", "xmm1", "xmm2", "xmm3", "xmm4", "xmm5");
-}
-
-#endif /* COGL_USE_PREMULT_SSE2 */
-
 gboolean
 _cogl_bitmap_fallback_can_convert (CoglPixelFormat src, CoglPixelFormat dst)
 {
@@ -342,12 +257,12 @@ _cogl_bitmap_fallback_convert (const CoglBitmap *bmp,
 			       CoglBitmap       *dst_bmp,
 			       CoglPixelFormat   dst_format)
 {
-  guint8  *src;
-  guint8  *dst;
-  int      src_bpp;
-  int      dst_bpp;
-  int      x,y;
-  guint8   temp_rgba[4] = {0,0,0,0};
+  guchar  *src;
+  guchar  *dst;
+  gint     src_bpp;
+  gint     dst_bpp;
+  gint     x,y;
+  guchar   temp_rgba[4] = {0,0,0,0};
 
   /* Make sure conversion supported */
   if (!_cogl_bitmap_fallback_can_convert (bmp->format, dst_format))
@@ -358,24 +273,20 @@ _cogl_bitmap_fallback_convert (const CoglBitmap *bmp,
 
   /* Initialize destination bitmap */
   *dst_bmp = *bmp;
-  dst_bmp->rowstride = sizeof(guint8) * dst_bpp * dst_bmp->width;
-  /* Copy the premult bit if the new format has an alpha channel */
-  if ((dst_format & COGL_A_BIT))
-    dst_bmp->format = ((bmp->format & COGL_PREMULT_BIT) |
-                       (dst_format & COGL_UNPREMULT_MASK));
-  else
-    dst_bmp->format = dst_format;
+  dst_bmp->rowstride = sizeof(guchar) * dst_bpp * dst_bmp->width;
+  dst_bmp->format = ((bmp->format & COGL_PREMULT_BIT) |
+		     (dst_format & COGL_UNPREMULT_MASK));
 
   /* Allocate a new buffer to hold converted data */
-  dst_bmp->data = g_malloc (sizeof(guint8)
+  dst_bmp->data = g_malloc (sizeof(guchar)
 			    * dst_bmp->height
 			    * dst_bmp->rowstride);
 
   /* FIXME: Optimize */
   for (y = 0; y < bmp->height; y++)
     {
-      src = (guint8*)bmp->data      + y * bmp->rowstride;
-      dst = (guint8*)dst_bmp->data  + y * dst_bmp->rowstride;
+      src = (guchar*)bmp->data      + y * bmp->rowstride;
+      dst = (guchar*)dst_bmp->data  + y * dst_bmp->rowstride;
 
       for (x = 0; x < bmp->width; x++)
 	{
@@ -433,8 +344,8 @@ _cogl_bitmap_fallback_convert (const CoglBitmap *bmp,
 gboolean
 _cogl_bitmap_fallback_unpremult (CoglBitmap *bmp)
 {
-  guint8  *p;
-  int      x,y;
+  guchar  *p;
+  gint     x,y;
 
   /* Make sure format supported for un-premultiplication */
   if (!_cogl_bitmap_fallback_can_unpremult (bmp->format))
@@ -442,7 +353,7 @@ _cogl_bitmap_fallback_unpremult (CoglBitmap *bmp)
 
   for (y = 0; y < bmp->height; y++)
     {
-      p = (guint8*) bmp->data + y * bmp->rowstride;
+      p = (guchar*) bmp->data + y * bmp->rowstride;
 
       if (bmp->format & COGL_AFIRST_BIT)
         {
@@ -476,8 +387,8 @@ _cogl_bitmap_fallback_unpremult (CoglBitmap *bmp)
 gboolean
 _cogl_bitmap_fallback_premult (CoglBitmap *bmp)
 {
-  guint8  *p;
-  int      x,y;
+  guchar  *p;
+  gint     x,y;
 
   /* Make sure format supported for un-premultiplication */
   if (!_cogl_bitmap_fallback_can_premult (bmp->format))
@@ -485,7 +396,7 @@ _cogl_bitmap_fallback_premult (CoglBitmap *bmp)
 
   for (y = 0; y < bmp->height; y++)
     {
-      p = (guint8*) bmp->data + y * bmp->rowstride;
+      p = (guchar*) bmp->data + y * bmp->rowstride;
 
       if (bmp->format & COGL_AFIRST_BIT)
         {
@@ -497,24 +408,7 @@ _cogl_bitmap_fallback_premult (CoglBitmap *bmp)
         }
       else
         {
-          x = bmp->width;
-
-#ifdef COGL_USE_PREMULT_SSE2
-
-          /* Process 4 pixels at a time */
-          while (x >= 4)
-            {
-              _cogl_premult_alpha_last_four_pixels_sse2 (p);
-              p += 4 * 4;
-              x -= 4;
-            }
-
-          /* If there are any pixels left we will fall through and
-             handle them below */
-
-#endif /* COGL_USE_PREMULT_SSE2 */
-
-          while (x-- > 0)
+          for (x = 0; x < bmp->width; x++)
             {
               _cogl_premult_alpha_last (p);
               p += 4;
@@ -529,7 +423,7 @@ _cogl_bitmap_fallback_premult (CoglBitmap *bmp)
 
 gboolean
 _cogl_bitmap_fallback_from_file (CoglBitmap  *bmp,
-				 const char  *filename)
+				 const gchar *filename)
 {
   /* FIXME: use jpeglib, libpng, etc. manually maybe */
   return FALSE;
