@@ -40,12 +40,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#include <clutter/clutter-stage.h>
+#include <clutter/clutter.h>
 #include <clutter/glx/clutter-glx-texture-pixmap.h>
 
-G_BEGIN_DECLS
-
-
-G_END_DECLS
-
 #endif /* __CLUTTER_GLX_H__ */
