@@ -102,24 +102,20 @@ clutter_stage_x11_fix_window_size (ClutterStageX11 *stage_x11,
 
   if (stage_x11->xwin != None && !stage_x11->is_foreign_xwin)
     {
-      gint min_width, min_height;
-      ClutterGeometry min_size;
+      guint min_width, min_height;
       XSizeHints *size_hints;
 
       size_hints = XAllocSizeHints();
 
-      _clutter_stage_window_get_geometry (CLUTTER_STAGE_WINDOW (stage_x11),
-                                          &min_size);
+      clutter_stage_get_minimum_size (stage_x11->wrapper,
+                                      &min_width,
+                                      &min_height);
 
-      if (new_width < 0)
-        min_width = min_size.width;
-      else
-        min_width = new_width;
+      if (new_width <= 0)
+        new_width = min_width;
 
-      if (new_height < 0)
-        min_height = min_size.height;
-      else
-        min_height = new_height;
+      if (new_height <= 0)
+        new_height = min_height;
 
       size_hints->flags = 0;
 
@@ -127,15 +123,19 @@ clutter_stage_x11_fix_window_size (ClutterStageX11 *stage_x11,
          restrictions on the window size */
       if (!stage_x11->fullscreen_on_map)
         {
-          size_hints->min_width = min_width;
-          size_hints->min_height = min_height;
-          size_hints->flags = PMinSize;
-
-          if (!resize)
+          if (resize)
             {
-              size_hints->max_width = size_hints->min_width;
-              size_hints->max_height = size_hints->min_height;
-              size_hints->flags |= PMaxSize;
+              size_hints->min_width = min_width;
+              size_hints->min_height = min_height;
+              size_hints->flags = PMinSize;
+            }
+          else
+            {
+              size_hints->min_width = new_width;
+              size_hints->min_height = new_height;
+              size_hints->max_width = new_width;
+              size_hints->max_height = new_height;
+              size_hints->flags = PMinSize | PMaxSize;
             }
         }
 
@@ -169,7 +169,7 @@ clutter_stage_x11_get_geometry (ClutterStageWindow *stage_window,
   ClutterBackend *backend = clutter_get_default_backend ();
   ClutterBackendX11 *backend_x11;
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
-  gboolean is_fullscreen, resize;
+  gboolean is_fullscreen;
 
   g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
@@ -187,19 +187,8 @@ clutter_stage_x11_get_geometry (ClutterStageWindow *stage_window,
       return;
     }
 
-  resize = clutter_stage_get_user_resizable (stage_x11->wrapper);
-
-  if (resize)
-    {
-      /* FIXME need API to set this */
-      geometry->width = 1;
-      geometry->height = 1;
-    }
-  else
-    {
-      geometry->width = stage_x11->xwin_width;
-      geometry->height = stage_x11->xwin_height;
-    }
+  geometry->width = stage_x11->xwin_width;
+  geometry->height = stage_x11->xwin_height;
 }
 
 static void
@@ -210,6 +199,7 @@ clutter_stage_x11_resize (ClutterStageWindow *stage_window,
   ClutterBackend *backend = clutter_get_default_backend ();
   ClutterBackendX11 *backend_x11;
   ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (stage_window);
+  ClutterStage *stage = stage_x11->wrapper;
   gboolean resize;
 
   resize = clutter_stage_get_user_resizable (stage_x11->wrapper);
@@ -250,6 +240,22 @@ clutter_stage_x11_resize (ClutterStageWindow *stage_window,
                          stage_x11->xwin,
                          stage_x11->xwin_width,
                          stage_x11->xwin_height);
+
+          /* If the viewport hasn't previously been initialized then even
+           * though we can't guarantee that the server will honour our request
+           * we need to ensure a valid viewport is set before our first paint.
+           */
+          if (G_UNLIKELY (!stage_x11->viewport_initialized))
+            {
+              ClutterPerspective perspective;
+              clutter_stage_get_perspective (stage, &perspective);
+              _cogl_setup_viewport (stage_x11->xwin_width,
+                                    stage_x11->xwin_height,
+                                    perspective.fovy,
+                                    perspective.aspect,
+                                    perspective.z_near,
+                                    perspective.z_far);
+            }
         }
 
       if (!resize)
@@ -508,16 +514,21 @@ clutter_stage_x11_set_user_resizable (ClutterStageWindow *stage_window,
 static void
 update_wm_hints (ClutterStageX11 *stage_x11)
 {
-  ClutterBackend *backend = clutter_get_default_backend ();
+  ClutterBackend *backend;
   ClutterBackendX11 *backend_x11;
   XWMHints wm_hints;
 
-  g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
-  backend_x11 = CLUTTER_BACKEND_X11 (backend);
-
   if (stage_x11->wm_state & STAGE_X11_WITHDRAWN)
     return;
 
+  if (stage_x11->is_foreign_xwin)
+    return;
+
+  backend = clutter_get_default_backend ();
+
+  g_return_if_fail (CLUTTER_IS_BACKEND_X11 (backend));
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
+
   wm_hints.flags = StateHint;
   wm_hints.initial_state = NormalState;
 
@@ -556,7 +567,7 @@ clutter_stage_x11_show (ClutterStageWindow *stage_window,
 
   if (stage_x11->xwin != None)
     {
-      if (do_raise)
+      if (do_raise && !stage_x11->is_foreign_xwin)
         {
           CLUTTER_NOTE (BACKEND, "Raising stage[%lu]",
                         (unsigned long) stage_x11->xwin);
@@ -582,7 +593,8 @@ clutter_stage_x11_show (ClutterStageWindow *stage_window,
 
       clutter_actor_map (CLUTTER_ACTOR (stage_x11->wrapper));
 
-      XMapWindow (backend_x11->xdpy, stage_x11->xwin);
+      if (!stage_x11->is_foreign_xwin)
+        XMapWindow (backend_x11->xdpy, stage_x11->xwin);
     }
 }
 
@@ -605,9 +617,8 @@ clutter_stage_x11_hide (ClutterStageWindow *stage_window)
 
       clutter_actor_unmap (CLUTTER_ACTOR (stage_x11->wrapper));
 
-      XWithdrawWindow (backend_x11->xdpy,
-                       stage_x11->xwin,
-                       0);
+      if (!stage_x11->is_foreign_xwin)
+        XWithdrawWindow (backend_x11->xdpy, stage_x11->xwin, 0);
     }
 }
 
@@ -654,6 +665,7 @@ clutter_stage_x11_init (ClutterStageX11 *stage)
   stage->is_foreign_xwin = FALSE;
   stage->fullscreen_on_map = FALSE;
   stage->is_cursor_visible = TRUE;
+  stage->viewport_initialized = FALSE;
 
   stage->title = NULL;
 
@@ -747,8 +759,10 @@ clutter_x11_get_stage_from_window (Window win)
  * along the lines of clutter_backend_x11_get_foreign_visual () or perhaps
  * clutter_stage_x11_get_foreign_visual ()
  *
- * Return value: An XVisualInfo suitable for creating a foreign stage.
- *               You should free this using XFree.
+ * Return value: An XVisualInfo suitable for creating a foreign stage. Use
+ *   XFree() to free the returned value instead
+ *
+ * Deprecated: 1.2: Use clutter_x11_get_visual_info() instead
  *
  * Since: 0.4
  */
